@@ -1,17 +1,32 @@
-from cattr import converters, disambiguators, dispatch, errors, gen, preconf
-
-from .converters import Converter, GenConverter, UnstructureStrategy
+from .converters import BaseConverter, Converter, GenConverter, UnstructureStrategy
+from .errors import (
+    AttributeValidationNote,
+    BaseValidationError,
+    ClassValidationError,
+    ForbiddenExtraKeysError,
+    IterableValidationError,
+    IterableValidationNote,
+    StructureHandlerNotFoundError,
+)
 from .gen import override
+from .v import transform_error
 
 __all__ = (
+    "AttributeValidationNote",
+    "BaseConverter",
+    "BaseValidationError",
+    "ClassValidationError",
     "Converter",
     "converters",
     "disambiguators",
     "dispatch",
     "errors",
+    "ForbiddenExtraKeysError",
     "gen",
     "GenConverter",
     "global_converter",
+    "IterableValidationError",
+    "IterableValidationNote",
     "override",
     "preconf",
     "register_structure_hook_func",
@@ -21,12 +36,14 @@ __all__ = (
     "structure_attrs_fromdict",
     "structure_attrs_fromtuple",
     "structure",
+    "StructureHandlerNotFoundError",
+    "transform_error",
     "unstructure",
     "UnstructureStrategy",
 )
 
 
-global_converter = GenConverter()
+global_converter = Converter()
 
 unstructure = global_converter.unstructure
 structure = global_converter.structure
