@@ -1,31 +1,47 @@
-from collections import Counter
+from collections import Counter, deque
 from collections.abc import MutableSet as AbcMutableSet
 from dataclasses import Field
 from enum import Enum
 from functools import lru_cache
-from typing import Any, Callable, Dict, Optional, Tuple, Type, TypeVar, Union
+from pathlib import Path
+from typing import (
+    Any,
+    Callable,
+    Deque,
+    Dict,
+    Iterable,
+    List,
+    NoReturn,
+    Optional,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+)
 
 from attr import Attribute
 from attr import has as attrs_has
 from attr import resolve_types
 
-from cattrs.errors import IterableValidationError, StructureHandlerNotFoundError
-
 from ._compat import (
     FrozenSetSubscriptable,
     Mapping,
     MutableMapping,
     MutableSequence,
-    MutableSet,
+    OriginAbstractSet,
+    OriginMutableSet,
     Sequence,
     Set,
     fields,
+    get_final_base,
+    get_newtype_base,
     get_origin,
     has,
     has_with_generic,
     is_annotated,
     is_bare,
     is_counter,
+    is_deque,
     is_frozenset,
     is_generic,
     is_generic_attrs,
@@ -36,12 +52,23 @@ from ._compat import (
     is_protocol,
     is_sequence,
     is_tuple,
+    is_typeddict,
     is_union_type,
 )
 from .disambiguators import create_uniq_field_dis_func
 from .dispatch import MultiStrategyDispatch
+from .errors import (
+    IterableValidationError,
+    IterableValidationNote,
+    StructureHandlerNotFoundError,
+)
 from .gen import (
     AttributeOverride,
+    DictStructureFn,
+    HeteroTupleUnstructureFn,
+    IterableUnstructureFn,
+    MappingStructureFn,
+    MappingUnstructureFn,
     make_dict_structure_fn,
     make_dict_unstructure_fn,
     make_hetero_tuple_unstructure_fn,
@@ -49,6 +76,8 @@ from .gen import (
     make_mapping_structure_fn,
     make_mapping_unstructure_fn,
 )
+from .gen.typeddicts import make_dict_structure_fn as make_typeddict_dict_struct_fn
+from .gen.typeddicts import make_dict_unstructure_fn as make_typeddict_dict_unstruct_fn
 
 NoneType = type(None)
 T = TypeVar("T")
@@ -62,30 +91,30 @@ class UnstructureStrategy(Enum):
     AS_TUPLE = "astuple"
 
 
-def _subclass(typ):
+def _subclass(typ: Type) -> Callable[[Type], bool]:
     """a shortcut"""
     return lambda cls: issubclass(cls, typ)
 
 
-def is_attrs_union(typ):
+def is_attrs_union(typ: Type) -> bool:
     return is_union_type(typ) and all(has(get_origin(e) or e) for e in typ.__args__)
 
 
-def is_attrs_union_or_none(typ):
+def is_attrs_union_or_none(typ: Type) -> bool:
     return is_union_type(typ) and all(
         e is NoneType or has(get_origin(e) or e) for e in typ.__args__
     )
 
 
-def is_optional(typ):
+def is_optional(typ: Type) -> bool:
     return is_union_type(typ) and NoneType in typ.__args__ and len(typ.__args__) == 2
 
 
-def is_literal_containing_enums(typ):
+def is_literal_containing_enums(typ: Type) -> bool:
     return is_literal(typ) and any(isinstance(val, Enum) for val in typ.__args__)
 
 
-class Converter:
+class BaseConverter:
     """Converts between structured and unstructured data."""
 
     __slots__ = (
@@ -124,7 +153,11 @@ class Converter:
 
         self._unstructure_func = MultiStrategyDispatch(self._unstructure_identity)
         self._unstructure_func.register_cls_list(
-            [(bytes, self._unstructure_identity), (str, self._unstructure_identity)]
+            [
+                (bytes, self._unstructure_identity),
+                (str, self._unstructure_identity),
+                (Path, str),
+            ]
         )
         self._unstructure_func.register_func_list(
             [
@@ -132,6 +165,11 @@ class Converter:
                     is_protocol,
                     lambda o: self.unstructure(o, unstructure_as=o.__class__),
                 ),
+                (
+                    lambda t: get_final_base(t) is not None,
+                    lambda t: self._unstructure_func.dispatch(get_final_base(t)),
+                    True,
+                ),
                 (is_mapping, self._unstructure_mapping),
                 (is_sequence, self._unstructure_seq),
                 (is_mutable_set, self._unstructure_seq),
@@ -145,14 +183,21 @@ class Converter:
         # Per-instance register of to-attrs converters.
         # Singledispatch dispatches based on the first argument, so we
         # store the function and switch the arguments in self.loads.
-        self._structure_func = MultiStrategyDispatch(self._structure_error)
+        self._structure_func = MultiStrategyDispatch(BaseConverter._structure_error)
         self._structure_func.register_func_list(
             [
                 (lambda cl: cl is Any or cl is Optional or cl is None, lambda v, _: v),
                 (is_generic_attrs, self._gen_structure_generic, True),
+                (lambda t: get_newtype_base(t) is not None, self._structure_newtype),
+                (
+                    lambda t: get_final_base(t) is not None,
+                    self._structure_final_factory,
+                    True,
+                ),
                 (is_literal, self._structure_simple_literal),
                 (is_literal_containing_enums, self._structure_enum_literal),
                 (is_sequence, self._structure_list),
+                (is_deque, self._structure_deque),
                 (is_mutable_set, self._structure_set),
                 (is_frozenset, self._structure_frozenset),
                 (is_tuple, self._structure_tuple),
@@ -174,6 +219,7 @@ class Converter:
                 (int, self._structure_call),
                 (float, self._structure_call),
                 (Enum, self._structure_call),
+                (Path, self._structure_call),
             ]
         )
 
@@ -182,7 +228,7 @@ class Converter:
         # Unions are instances now, not classes. We use different registries.
         self._union_struct_registry: Dict[Any, Callable[[Any, Type[T]], T]] = {}
 
-    def unstructure(self, obj: Any, unstructure_as=None) -> Any:
+    def unstructure(self, obj: Any, unstructure_as: Any = None) -> Any:
         return self._unstructure_func.dispatch(
             obj.__class__ if unstructure_as is None else unstructure_as
         )(obj)
@@ -196,7 +242,7 @@ class Converter:
             else UnstructureStrategy.AS_TUPLE
         )
 
-    def register_unstructure_hook(self, cls: Any, func: Callable[[T], Any]) -> None:
+    def register_unstructure_hook(self, cls: Any, func: Callable[[Any], Any]) -> None:
         """Register a class-to-primitive converter function for a class.
 
         The converter function should take an instance of the class and return
@@ -206,12 +252,15 @@ class Converter:
             resolve_types(cls)
         if is_union_type(cls):
             self._unstructure_func.register_func_list([(lambda t: t == cls, func)])
+        elif get_newtype_base(cls) is not None:
+            # This is a newtype, so we handle it specially.
+            self._unstructure_func.register_func_list([(lambda t: t is cls, func)])
         else:
             self._unstructure_func.register_cls_list([(cls, func)])
 
     def register_unstructure_hook_func(
-        self, check_func: Callable[[Any], bool], func: Callable[[T], Any]
-    ):
+        self, check_func: Callable[[Any], bool], func: Callable[[Any], Any]
+    ) -> None:
         """Register a class-to-primitive converter function for a class, using
         a function to check if it's a match.
         """
@@ -233,7 +282,9 @@ class Converter:
         """
         self._unstructure_func.register_func_list([(predicate, factory, True)])
 
-    def register_structure_hook(self, cl: Any, func: Callable[[Any, Type[T]], T]):
+    def register_structure_hook(
+        self, cl: Any, func: Callable[[Any, Type[T]], T]
+    ) -> None:
         """Register a primitive-to-class converter function for a type.
 
         The converter function should take two arguments:
@@ -248,12 +299,15 @@ class Converter:
         if is_union_type(cl):
             self._union_struct_registry[cl] = func
             self._structure_func.clear_cache()
+        elif get_newtype_base(cl) is not None:
+            # This is a newtype, so we handle it specially.
+            self._structure_func.register_func_list([(lambda t: t is cl, func)])
         else:
             self._structure_func.register_cls_list([(cl, func)])
 
     def register_structure_hook_func(
         self, check_func: Callable[[Type[T]], bool], func: Callable[[Any, Type[T]], T]
-    ):
+    ) -> None:
         """Register a class-to-primitive converter function for a class, using
         a function to check if it's a match.
         """
@@ -262,7 +316,7 @@ class Converter:
     def register_structure_hook_factory(
         self,
         predicate: Callable[[Any], bool],
-        factory: Callable[[Any], Callable[[Any], Any]],
+        factory: Callable[[Any], Callable[[Any, Any], Any]],
     ) -> None:
         """
         Register a hook factory for a given predicate.
@@ -277,11 +331,10 @@ class Converter:
 
     def structure(self, obj: Any, cl: Type[T]) -> T:
         """Convert unstructured Python data structures to structured data."""
-
         return self._structure_func.dispatch(cl)(obj, cl)
 
     # Classes to Python primitives.
-    def unstructure_attrs_asdict(self, obj) -> Dict[str, Any]:
+    def unstructure_attrs_asdict(self, obj: Any) -> Dict[str, Any]:
         """Our version of `attrs.asdict`, so we can call back to us."""
         attrs = fields(obj.__class__)
         dispatch = self._unstructure_func.dispatch
@@ -292,7 +345,7 @@ class Converter:
             rv[name] = dispatch(a.type or v.__class__)(v)
         return rv
 
-    def unstructure_attrs_astuple(self, obj) -> Tuple[Any, ...]:
+    def unstructure_attrs_astuple(self, obj: Any) -> Tuple[Any, ...]:
         """Our version of `attrs.astuple`, so we can call back to us."""
         attrs = fields(obj.__class__)
         dispatch = self._unstructure_func.dispatch
@@ -303,21 +356,22 @@ class Converter:
             res.append(dispatch(a.type or v.__class__)(v))
         return tuple(res)
 
-    def _unstructure_enum(self, obj):
+    def _unstructure_enum(self, obj: Enum) -> Any:
         """Convert an enum to its value."""
         return obj.value
 
-    def _unstructure_identity(self, obj):
+    @staticmethod
+    def _unstructure_identity(obj: T) -> T:
         """Just pass it through."""
         return obj
 
-    def _unstructure_seq(self, seq):
+    def _unstructure_seq(self, seq: Sequence[T]) -> Sequence[T]:
         """Convert a sequence to primitive equivalents."""
         # We can reuse the sequence class, so tuples stay tuples.
         dispatch = self._unstructure_func.dispatch
         return seq.__class__(dispatch(e.__class__)(e) for e in seq)
 
-    def _unstructure_mapping(self, mapping):
+    def _unstructure_mapping(self, mapping: Mapping[T, V]) -> Mapping[T, V]:
         """Convert a mapping of attr classes to primitive equivalents."""
 
         # We can reuse the mapping class, so dicts stay dicts and OrderedDicts
@@ -328,7 +382,10 @@ class Converter:
             for k, v in mapping.items()
         )
 
-    def _unstructure_union(self, obj):
+    # note: Use UnionType when 3.11 is released as
+    # the behaviour of @final is changed. This would
+    # affect how we can support UnionType in ._compat.py
+    def _unstructure_union(self, obj: Any) -> Any:
         """
         Unstructure an object as a union.
 
@@ -338,19 +395,22 @@ class Converter:
 
     # Python primitives to classes.
 
-    def _structure_error(self, _, cl):
+    @staticmethod
+    def _structure_error(_, cl: Type) -> NoReturn:
         """At the bottom of the condition stack, we explode if we can't handle it."""
         msg = "Unsupported type: {0!r}. Register a structure hook for " "it.".format(cl)
         raise StructureHandlerNotFoundError(msg, type_=cl)
 
-    def _gen_structure_generic(self, cl):
+    def _gen_structure_generic(self, cl: Type[T]) -> DictStructureFn[T]:
         """Create and return a hook for structuring generics."""
         fn = make_dict_structure_fn(
             cl, self, _cattrs_prefer_attrib_converters=self._prefer_attrib_converters
         )
         return fn
 
-    def _gen_attrs_union_structure(self, cl):
+    def _gen_attrs_union_structure(
+        self, cl: Any
+    ) -> Callable[[Any, Type[T]], Optional[Type[T]]]:
         """Generate a structuring function for a union of attrs classes (and maybe None)."""
         dis_fn = self._get_dis_func(cl)
         has_none = NoneType in cl.__args__
@@ -370,7 +430,7 @@ class Converter:
         return structure_attrs_union
 
     @staticmethod
-    def _structure_call(obj, cl):
+    def _structure_call(obj: Any, cl: Type[T]) -> Any:
         """Just call ``cl`` with the given ``obj``.
 
         This is just an optimization on the ``_structure_default`` case, when
@@ -393,17 +453,29 @@ class Converter:
         except KeyError:
             raise Exception(f"{val} not in literal {type}") from None
 
+    def _structure_newtype(self, val, type):
+        base = get_newtype_base(type)
+        return self._structure_func.dispatch(base)(val, base)
+
+    def _structure_final_factory(self, type):
+        base = get_final_base(type)
+        res = self._structure_func.dispatch(base)
+        if res == self._structure_call:
+            # It's not really `structure_call` for Finals (can't call Final())
+            return lambda v, _: self._structure_call(v, base)
+        return res
+
     # Attrs classes.
 
     def structure_attrs_fromtuple(self, obj: Tuple[Any, ...], cl: Type[T]) -> T:
         """Load an attrs class from a sequence (tuple)."""
         conv_obj = []  # A list of converter parameters.
-        for a, value in zip(fields(cl), obj):  # type: ignore
+        for a, value in zip(fields(cl), obj):
             # We detect the type by the metadata.
             converted = self._structure_attribute(a, value)
             conv_obj.append(converted)
 
-        return cl(*conv_obj)  # type: ignore
+        return cl(*conv_obj)
 
     def _structure_attribute(self, a: Union[Attribute, Field], value: Any) -> Any:
         """Handle an individual attrs attribute."""
@@ -432,7 +504,7 @@ class Converter:
         # For public use.
 
         conv_obj = {}  # Start with a fresh dict, to ignore extra keys.
-        for a in fields(cl):  # type: ignore
+        for a in fields(cl):
             name = a.name
 
             try:
@@ -445,9 +517,9 @@ class Converter:
 
             conv_obj[name] = self._structure_attribute(a, val)
 
-        return cl(**conv_obj)  # type: ignore
+        return cl(**conv_obj)
 
-    def _structure_list(self, obj, cl):
+    def _structure_list(self, obj: Iterable[T], cl: Any) -> List[T]:
         """Convert an iterable to a potentially generic list."""
         if is_bare(cl) or cl.__args__[0] is Any:
             res = [e for e in obj]
@@ -462,7 +534,10 @@ class Converter:
                     try:
                         res.append(handler(e, elem_type))
                     except Exception as e:
-                        e.__note__ = f"Structuring {cl} @ index {ix}"
+                        msg = IterableValidationNote(
+                            f"Structuring {cl} @ index {ix}", ix, elem_type
+                        )
+                        e.__notes__ = getattr(e, "__notes__", []) + [msg]
                         errors.append(e)
                     finally:
                         ix += 1
@@ -474,7 +549,39 @@ class Converter:
                 res = [handler(e, elem_type) for e in obj]
         return res
 
-    def _structure_set(self, obj, cl, structure_to=set):
+    def _structure_deque(self, obj: Iterable[T], cl: Any) -> Deque[T]:
+        """Convert an iterable to a potentially generic deque."""
+        if is_bare(cl) or cl.__args__[0] is Any:
+            res = deque(e for e in obj)
+        else:
+            elem_type = cl.__args__[0]
+            handler = self._structure_func.dispatch(elem_type)
+            if self.detailed_validation:
+                errors = []
+                res = deque()
+                ix = 0  # Avoid `enumerate` for performance.
+                for e in obj:
+                    try:
+                        res.append(handler(e, elem_type))
+                    except Exception as e:
+                        msg = IterableValidationNote(
+                            f"Structuring {cl} @ index {ix}", ix, elem_type
+                        )
+                        e.__notes__ = getattr(e, "__notes__", []) + [msg]
+                        errors.append(e)
+                    finally:
+                        ix += 1
+                if errors:
+                    raise IterableValidationError(
+                        f"While structuring {cl!r}", errors, cl
+                    )
+            else:
+                res = deque(handler(e, elem_type) for e in obj)
+        return res
+
+    def _structure_set(
+        self, obj: Iterable[T], cl: Any, structure_to: type = set
+    ) -> Set[T]:
         """Convert an iterable into a potentially generic set."""
         if is_bare(cl) or cl.__args__[0] is Any:
             return structure_to(obj)
@@ -483,14 +590,20 @@ class Converter:
         if self.detailed_validation:
             errors = []
             res = set()
+            ix = 0
             for e in obj:
                 try:
                     res.add(handler(e, elem_type))
                 except Exception as exc:
-                    exc.__note__ = (
-                        f"Structuring {structure_to.__name__} @ element {e!r}"
+                    msg = IterableValidationNote(
+                        f"Structuring {structure_to.__name__} @ element {e!r}",
+                        ix,
+                        elem_type,
                     )
+                    exc.__notes__ = getattr(e, "__notes__", []) + [msg]
                     errors.append(exc)
+                finally:
+                    ix += 1
             if errors:
                 raise IterableValidationError(f"While structuring {cl!r}", errors, cl)
             return res if structure_to is set else structure_to(res)
@@ -499,11 +612,13 @@ class Converter:
         else:
             return structure_to([handler(e, elem_type) for e in obj])
 
-    def _structure_frozenset(self, obj, cl):
+    def _structure_frozenset(
+        self, obj: Iterable[T], cl: Any
+    ) -> FrozenSetSubscriptable[T]:
         """Convert an iterable into a potentially generic frozenset."""
         return self._structure_set(obj, cl, structure_to=frozenset)
 
-    def _structure_dict(self, obj, cl):
+    def _structure_dict(self, obj: Mapping[T, V], cl: Any) -> Dict[T, V]:
         """Convert a mapping into a potentially generic dict."""
         if is_bare(cl) or cl.__args__ == (Any, Any):
             return dict(obj)
@@ -535,7 +650,7 @@ class Converter:
         handler = self._union_struct_registry[union]
         return handler(obj, union)
 
-    def _structure_tuple(self, obj, tup: Type[T]) -> T:
+    def _structure_tuple(self, obj: Any, tup: Type[T]) -> T:
         """Deal with structuring into a tuple."""
         if tup in (Tuple, tuple):
             tup_params = None
@@ -552,12 +667,18 @@ class Converter:
             if self.detailed_validation:
                 errors = []
                 res = []
-                for ix, e in enumerate(obj):
+                ix = 0
+                for e in obj:
                     try:
                         res.append(conv(e, tup_type))
                     except Exception as exc:
-                        exc.__note__ = f"Structuring {tup} @ index {ix}"
+                        msg = IterableValidationNote(
+                            f"Structuring {tup} @ index {ix}", ix, tup_type
+                        )
+                        exc.__notes__ = getattr(e, "__notes__", []) + [msg]
                         errors.append(exc)
+                    finally:
+                        ix += 1
                 if errors:
                     raise IterableValidationError(
                         f"While structuring {tup!r}", errors, tup
@@ -567,6 +688,14 @@ class Converter:
                 return tuple(conv(e, tup_type) for e in obj)
         else:
             # We're dealing with a heterogenous tuple.
+            exp_len = len(tup_params)
+            try:
+                len_obj = len(obj)
+            except TypeError:
+                pass  # most likely an unsized iterator, eg generator
+            else:
+                if len_obj > exp_len:
+                    exp_len = len_obj
             if self.detailed_validation:
                 errors = []
                 res = []
@@ -575,23 +704,40 @@ class Converter:
                         conv = self._structure_func.dispatch(t)
                         res.append(conv(e, t))
                     except Exception as exc:
-                        exc.__note__ = f"Structuring {tup} @ index {ix}"
+                        msg = IterableValidationNote(
+                            f"Structuring {tup} @ index {ix}", ix, t
+                        )
+                        exc.__notes__ = getattr(e, "__notes__", []) + [msg]
                         errors.append(exc)
+                if len(res) < exp_len:
+                    problem = "Not enough" if len(res) < len(tup_params) else "Too many"
+                    exc = ValueError(
+                        f"{problem} values in {obj!r} to structure as {tup!r}"
+                    )
+                    msg = f"Structuring {tup}"
+                    exc.__notes__ = getattr(e, "__notes__", []) + [msg]
+                    errors.append(exc)
                 if errors:
                     raise IterableValidationError(
                         f"While structuring {tup!r}", errors, tup
                     )
                 return tuple(res)
             else:
-                return tuple(
+                res = tuple(
                     [
                         self._structure_func.dispatch(t)(e, t)
                         for t, e in zip(tup_params, obj)
                     ]
                 )
+                if len(res) < exp_len:
+                    problem = "Not enough" if len(res) < len(tup_params) else "Too many"
+                    raise ValueError(
+                        f"{problem} values in {obj!r} to structure as {tup!r}"
+                    )
+                return res
 
     @staticmethod
-    def _get_dis_func(union) -> Callable[..., Type]:
+    def _get_dis_func(union: Any) -> Callable[[Any], Type]:
         """Fetch or try creating a disambiguation function for a union."""
         union_types = union.__args__
         if NoneType in union_types:  # type: ignore
@@ -609,8 +755,40 @@ class Converter:
             )
         return create_uniq_field_dis_func(*union_types)
 
+    def __deepcopy__(self, _) -> "BaseConverter":
+        return self.copy()
 
-class GenConverter(Converter):
+    def copy(
+        self,
+        dict_factory: Optional[Callable[[], Any]] = None,
+        unstruct_strat: Optional[UnstructureStrategy] = None,
+        prefer_attrib_converters: Optional[bool] = None,
+        detailed_validation: Optional[bool] = None,
+    ) -> "BaseConverter":
+        res = self.__class__(
+            dict_factory if dict_factory is not None else self._dict_factory,
+            unstruct_strat
+            if unstruct_strat is not None
+            else (
+                UnstructureStrategy.AS_DICT
+                if self._unstructure_attrs == self.unstructure_attrs_asdict
+                else UnstructureStrategy.AS_TUPLE
+            ),
+            prefer_attrib_converters
+            if prefer_attrib_converters is not None
+            else self._prefer_attrib_converters,
+            detailed_validation
+            if detailed_validation is not None
+            else self.detailed_validation,
+        )
+
+        self._unstructure_func.copy_to(res._unstructure_func)
+        self._structure_func.copy_to(res._structure_func)
+
+        return res
+
+
+class Converter(BaseConverter):
     """A converter which generates specialized un/structuring functions."""
 
     __slots__ = (
@@ -618,6 +796,8 @@ class GenConverter(Converter):
         "forbid_extra_keys",
         "type_overrides",
         "_unstruct_collection_overrides",
+        "_struct_copy_skip",
+        "_unstruct_copy_skip",
     )
 
     def __init__(
@@ -641,23 +821,27 @@ class GenConverter(Converter):
         self.forbid_extra_keys = forbid_extra_keys
         self.type_overrides = dict(type_overrides)
 
+        unstruct_collection_overrides = {
+            get_origin(k) or k: v for k, v in unstruct_collection_overrides.items()
+        }
+
         self._unstruct_collection_overrides = unstruct_collection_overrides
 
         # Do a little post-processing magic to make things easier for users.
         co = unstruct_collection_overrides
 
         # abc.Set overrides, if defined, apply to abc.MutableSets and sets
-        if Set in co:
-            if MutableSet not in co:
-                co[MutableSet] = co[Set]
-                co[AbcMutableSet] = co[Set]  # For 3.7/3.8 compatibility.
+        if OriginAbstractSet in co:
+            if OriginMutableSet not in co:
+                co[OriginMutableSet] = co[OriginAbstractSet]
+                co[AbcMutableSet] = co[OriginAbstractSet]  # For 3.7/3.8 compatibility.
             if FrozenSetSubscriptable not in co:
-                co[FrozenSetSubscriptable] = co[Set]
+                co[FrozenSetSubscriptable] = co[OriginAbstractSet]
 
         # abc.MutableSet overrrides, if defined, apply to sets
-        if MutableSet in co:
+        if OriginMutableSet in co:
             if set not in co:
-                co[set] = co[MutableSet]
+                co[set] = co[OriginMutableSet]
 
         if FrozenSetSubscriptable in co:
             co[frozenset] = co[FrozenSetSubscriptable]  # For 3.7/3.8 compatibility.
@@ -673,6 +857,8 @@ class GenConverter(Converter):
         if MutableSequence in co:
             if list not in co:
                 co[list] = co[MutableSequence]
+            if deque not in co:
+                co[deque] = co[MutableSequence]
 
         # abc.Mapping overrides, if defined, can apply to MutableMappings
         if Mapping in co:
@@ -715,9 +901,29 @@ class GenConverter(Converter):
             is_frozenset,
             lambda cl: self.gen_unstructure_iterable(cl, unstructure_to=frozenset),
         )
+        self.register_unstructure_hook_factory(
+            is_typeddict, self.gen_unstructure_typeddict
+        )
+        self.register_unstructure_hook_factory(
+            lambda t: get_newtype_base(t) is not None,
+            lambda t: self._unstructure_func.dispatch(get_newtype_base(t)),
+        )
         self.register_structure_hook_factory(is_annotated, self.gen_structure_annotated)
         self.register_structure_hook_factory(is_mapping, self.gen_structure_mapping)
         self.register_structure_hook_factory(is_counter, self.gen_structure_counter)
+        self.register_structure_hook_factory(is_typeddict, self.gen_structure_typeddict)
+        self.register_structure_hook_factory(
+            lambda t: get_newtype_base(t) is not None, self.get_structure_newtype
+        )
+
+        # We keep these so we can more correctly copy the hooks.
+        self._struct_copy_skip = self._structure_func.get_num_fns()
+        self._unstruct_copy_skip = self._unstructure_func.get_num_fns()
+
+    def get_structure_newtype(self, type: Type[T]) -> Callable[[Any, Any], T]:
+        base = get_newtype_base(type)
+        handler = self._structure_func.dispatch(base)
+        return lambda v, _: handler(v, base)
 
     def gen_unstructure_annotated(self, type):
         origin = type.__origin__
@@ -729,7 +935,16 @@ class GenConverter(Converter):
         h = self._structure_func.dispatch(origin)
         return h
 
-    def gen_unstructure_attrs_fromdict(self, cl: Type[T]) -> Dict[str, Any]:
+    def gen_unstructure_typeddict(self, cl: Any) -> Callable[[Dict], Dict]:
+        """Generate a TypedDict unstructure function.
+
+        Also apply converter-scored modifications.
+        """
+        return make_typeddict_dict_unstruct_fn(cl, self)
+
+    def gen_unstructure_attrs_fromdict(
+        self, cl: Type[T]
+    ) -> Callable[[T], Dict[str, Any]]:
         origin = get_origin(cl)
         attribs = fields(origin or cl)
         if attrs_has(cl) and any(isinstance(a.type, str) for a in attribs):
@@ -746,8 +961,19 @@ class GenConverter(Converter):
         )
         return h
 
-    def gen_structure_attrs_fromdict(self, cl: Type[T]) -> T:
-        attribs = fields(get_origin(cl) if is_generic(cl) else cl)
+    def gen_structure_typeddict(self, cl: Any) -> Callable[[Dict], Dict]:
+        """Generate a TypedDict structure function.
+
+        Also apply converter-scored modifications.
+        """
+        return make_typeddict_dict_struct_fn(
+            cl, self, _cattrs_detailed_validation=self.detailed_validation
+        )
+
+    def gen_structure_attrs_fromdict(
+        self, cl: Type[T]
+    ) -> Callable[[Mapping[str, Any], Any], T]:
+        attribs = fields(get_origin(cl) or cl if is_generic(cl) else cl)
         if attrs_has(cl) and any(isinstance(a.type, str) for a in attribs):
             # PEP 563 annotations - need to be resolved.
             resolve_types(cl)
@@ -767,7 +993,9 @@ class GenConverter(Converter):
         # only direct dispatch so that subclasses get separately generated
         return h
 
-    def gen_unstructure_iterable(self, cl: Any, unstructure_to=None):
+    def gen_unstructure_iterable(
+        self, cl: Any, unstructure_to: Any = None
+    ) -> IterableUnstructureFn:
         unstructure_to = self._unstruct_collection_overrides.get(
             get_origin(cl) or cl, unstructure_to or list
         )
@@ -775,7 +1003,9 @@ class GenConverter(Converter):
         self._unstructure_func.register_cls_list([(cl, h)], direct=True)
         return h
 
-    def gen_unstructure_hetero_tuple(self, cl: Any, unstructure_to=None):
+    def gen_unstructure_hetero_tuple(
+        self, cl: Any, unstructure_to: Any = None
+    ) -> HeteroTupleUnstructureFn:
         unstructure_to = self._unstruct_collection_overrides.get(
             get_origin(cl) or cl, unstructure_to or list
         )
@@ -783,7 +1013,12 @@ class GenConverter(Converter):
         self._unstructure_func.register_cls_list([(cl, h)], direct=True)
         return h
 
-    def gen_unstructure_mapping(self, cl: Any, unstructure_to=None, key_handler=None):
+    def gen_unstructure_mapping(
+        self,
+        cl: Any,
+        unstructure_to: Any = None,
+        key_handler: Optional[Callable[[Any, Optional[Any]], Any]] = None,
+    ) -> MappingUnstructureFn:
         unstructure_to = self._unstruct_collection_overrides.get(
             get_origin(cl) or cl, unstructure_to or dict
         )
@@ -793,12 +1028,66 @@ class GenConverter(Converter):
         self._unstructure_func.register_cls_list([(cl, h)], direct=True)
         return h
 
-    def gen_structure_counter(self, cl: Any):
-        h = make_mapping_structure_fn(cl, self, structure_to=Counter, val_type=int)
+    def gen_structure_counter(self, cl: Any) -> MappingStructureFn[T]:
+        h = make_mapping_structure_fn(
+            cl,
+            self,
+            structure_to=Counter,
+            val_type=int,
+            detailed_validation=self.detailed_validation,
+        )
         self._structure_func.register_cls_list([(cl, h)], direct=True)
         return h
 
-    def gen_structure_mapping(self, cl: Any):
-        h = make_mapping_structure_fn(cl, self)
+    def gen_structure_mapping(self, cl: Any) -> MappingStructureFn[T]:
+        h = make_mapping_structure_fn(
+            cl, self, detailed_validation=self.detailed_validation
+        )
         self._structure_func.register_cls_list([(cl, h)], direct=True)
         return h
+
+    def copy(
+        self,
+        dict_factory: Optional[Callable[[], Any]] = None,
+        unstruct_strat: Optional[UnstructureStrategy] = None,
+        omit_if_default: Optional[bool] = None,
+        forbid_extra_keys: Optional[bool] = None,
+        type_overrides: Optional[Mapping[Type, AttributeOverride]] = None,
+        unstruct_collection_overrides: Optional[Mapping[Type, Callable]] = None,
+        prefer_attrib_converters: Optional[bool] = None,
+        detailed_validation: Optional[bool] = None,
+    ) -> "Converter":
+        res = self.__class__(
+            dict_factory if dict_factory is not None else self._dict_factory,
+            unstruct_strat
+            if unstruct_strat is not None
+            else (
+                UnstructureStrategy.AS_DICT
+                if self._unstructure_attrs == self.unstructure_attrs_asdict
+                else UnstructureStrategy.AS_TUPLE
+            ),
+            omit_if_default if omit_if_default is not None else self.omit_if_default,
+            forbid_extra_keys
+            if forbid_extra_keys is not None
+            else self.forbid_extra_keys,
+            type_overrides if type_overrides is not None else self.type_overrides,
+            unstruct_collection_overrides
+            if unstruct_collection_overrides is not None
+            else self._unstruct_collection_overrides,
+            prefer_attrib_converters
+            if prefer_attrib_converters is not None
+            else self._prefer_attrib_converters,
+            detailed_validation
+            if detailed_validation is not None
+            else self.detailed_validation,
+        )
+
+        self._unstructure_func.copy_to(
+            res._unstructure_func, skip=self._unstruct_copy_skip
+        )
+        self._structure_func.copy_to(res._structure_func, skip=self._struct_copy_skip)
+
+        return res
+
+
+GenConverter = Converter
